//+------------------------------------------------------------------+
//|                                               premier stochastic |
//|                                                           mladen |
//|                                                                  |
//| Based on Lee Leibfarth article in TASC August 2008               |
//| article : "Premier Stochastic Oscillator."                       |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers  5
#property indicator_color1   LimeGreen
#property indicator_color2   LimeGreen
#property indicator_color3   DarkOrange
#property indicator_color4   DarkOrange
#property indicator_color5   DimGray
#property indicator_width1   2
#property indicator_width3   2
#property indicator_width5   2
#property indicator_maximum  1
#property indicator_minimum -1
#property indicator_levelcolor DarkGray

//
//
//
//
//

extern string TimeFrame        = "Current time frame";
extern int    StochasticLength = 32;
extern int    StochasticMode   =  0;
extern int    SmoothLength     = 25;
extern double Level1           = 0.9;
extern double Level2           = 0.2;

//
//
//
//
//

double BufferUu[];
double BufferUd[];
double BufferDd[];
double BufferDu[];
double Buffer[];
double averages[][3];

int      timeFrame;
bool     returnBars;
string   indicatorFileName;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,BufferUu); SetIndexStyle(0,DRAW_HISTOGRAM);
   SetIndexBuffer(1,BufferUd); SetIndexStyle(1,DRAW_HISTOGRAM);
   SetIndexBuffer(2,BufferDd); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,BufferDu); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,Buffer);
      SetLevelValue(0, Level1);
      SetLevelValue(1, Level2);
      SetLevelValue(2, 0.00);
      SetLevelValue(3,-Level2);
      SetLevelValue(4,-Level1);

         timeFrame         = stringToTimeFrame(TimeFrame);
         indicatorFileName = WindowExpertName();
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);

   return(0);
}
int deinit()
{
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int start()
{
   int counted_bars = IndicatorCounted();
   int limit,i,r;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { BufferUu[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (timeFrame==Period())
   {
      if (ArrayRange(averages,0) != Bars) ArrayResize(averages,Bars);
      double alpha = 2.0/(MathSqrt(SmoothLength)+1.0);
      for(i = limit, r=Bars-i-1; i >= 0 ; i--, r++)
      {
         averages[r][0] = 0.1*(iStochastic(NULL,0,StochasticLength,1,1,MODE_SMA,StochasticMode,MODE_MAIN,i)-50);
         averages[r][1] = averages[r-1][1]+alpha*(averages[r][0]-averages[r-1][1]);
         averages[r][2] = averages[r-1][2]+alpha*(averages[r][1]-averages[r-1][2]);

         double exp  = MathExp(averages[r][2]);
         
         //
         //
         //
         //
         // 
      
         Buffer[i] = (exp-1)/(exp+1);
            BufferUu[i] = EMPTY_VALUE;
            BufferUd[i] = EMPTY_VALUE;
            BufferDd[i] = EMPTY_VALUE;
            BufferDu[i] = EMPTY_VALUE;
            if (Buffer[i] > 0)
            {
               if (Buffer[i] > Level1) { BufferUu[i] = Buffer[i]; continue; }
               if (Buffer[i] > Level2) { BufferUd[i] = Buffer[i]; continue; }
            }
            if (Buffer[i] < 0)
            {
               if (Buffer[i] < -Level1) { BufferDd[i] = Buffer[i]; continue;}
               if (Buffer[i] < -Level2) { BufferDu[i] = Buffer[i]; }
            }
      }
      return(0);
   }

   //
   //
   //
   //
   //

   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         Buffer[i]   = iCustom(NULL,timeFrame,indicatorFileName,"CalculateValue",StochasticLength,StochasticMode,SmoothLength,Level1,Level2,4,y);
         BufferUu[i] = EMPTY_VALUE;
         BufferUd[i] = EMPTY_VALUE;
         BufferDd[i] = EMPTY_VALUE;
         BufferDu[i] = EMPTY_VALUE;
            if (Buffer[i] > 0)
            {
               if (Buffer[i] > Level1) { BufferUu[i] = Buffer[i]; continue; }
               if (Buffer[i] > Level2) { BufferUd[i] = Buffer[i]; continue; }
            }
            if (Buffer[i] < 0)
            {
               if (Buffer[i] < -Level1) { BufferDd[i] = Buffer[i]; continue;}
               if (Buffer[i] < -Level2) { BufferDu[i] = Buffer[i]; }
            }
   }
   return(0);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}